﻿using FTD2XX_NET;

using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace PCL6115_EV2S
{
    public class accessPCL6115
    {
        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// クラス／テーブル
        /// Class / Table
        /// </summary>
        private clsFTDI hFTDI = new clsFTDI();

        FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

        private const int BUFFER_MAX = 1024;
        private byte[] PileUpBuffer = new byte[BUFFER_MAX];
        private int PileUpindex = 0;
        private bool BuffOver = false;
        public bool PCL6115EV_on = false;
        public bool PCL6115EV_stop = false;

        ////////////////////////////////////////////////////////////////////
        // 
        // PCL6115 レジスタ･リードコマンド --------------------------------
        // PCL6115 Register: Read Command --------------------------------
        public byte RRMV = 0xD0;
        public byte RRFL = 0xD1;
        public byte RRFH = 0xD2;
        public byte RRUR = 0xD3;
        public byte RRDR = 0xD4;
        public byte RRMG = 0xD5;
        public byte RRDP = 0xD6;
        public byte RRMD = 0xD7;
        public byte RRIP = 0xD8;
        public byte RRUS = 0xD9;
        public byte RRDS = 0xDA;
        public byte RRENV1 = 0xDC;
        public byte RRENV2 = 0xDD;
        public byte RRENV3 = 0xDE;
        public byte RRENV4 = 0xDF;
        public byte RRCUN1 = 0xE3;
        public byte RRCUN2 = 0xE4;
        public byte RRCMP1 = 0xE7;
        public byte RRCMP2 = 0xE8;
        public byte RRCMP3 = 0xE9;
        public byte RRCMP4 = 0xEA;
        public byte RRIRQ = 0xEC;
        public byte RRLTC1 = 0xED;
        public byte RRLTC2 = 0xEE;
        public byte RRLTC3 = 0xEF;
        public byte RRLTC4 = 0xF0;
        public byte RRSTS = 0xF1;
        public byte RREST = 0xF2;
        public byte RRIST = 0xF3;
        public byte RRPLS = 0xF4;
        public byte RRSPD = 0xF5;
        public byte RRSDC = 0xF6;
        public byte RPRMV = 0xC0;
        public byte RPRFL = 0xC1;
        public byte RPRFH = 0xC2;
        public byte RPRUR = 0xC3;
        public byte RPRDR = 0xC4;
        public byte RPRMG = 0xC5;
        public byte RPRDP = 0xC6;
        public byte RPRMD = 0xC7;
        public byte RPRIP = 0xC8;
        public byte RPRUS = 0xC9;
        public byte RPRDS = 0xCA;
        public byte RRGPM = 0xFA;
        public byte RRGPD = 0xFB;
        // PCL6115 レジスタ･ライトコマンド --------------------------------
        // PCL6115 Register: Write Command --------------------------------
        public byte WRMV = 0x90;
        public byte WRFL = 0x91;
        public byte WRFH = 0x92;
        public byte WRUR = 0x93;
        public byte WRDR = 0x94;
        public byte WRMG = 0x95;
        public byte WRDP = 0x96;
        public byte WRMD = 0x97;
        public byte WRIP = 0x98;
        public byte WRUS = 0x99;
        public byte WRDS = 0x9A;
        public byte WRENV1 = 0x9C;
        public byte WRENV2 = 0x9D;
        public byte WRENV3 = 0x9E;
        public byte WRENV4 = 0x9F;
        public byte WRCUN1 = 0xA3;
        public byte WRCUN2 = 0xA4;
        public byte WRCMP1 = 0xA7;
        public byte WRCMP2 = 0xA8;
        public byte WRCMP3 = 0xA9;
        public byte WRCMP4 = 0xAA;
        public byte WRIRQ = 0xAC;
        //public byte WRLTC1	= 0x00;
        //public byte WRLTC2	= 0x00;
        //public byte WRLTC3	= 0x00;
        //public byte WRLTC4	= 0x00;
        //public byte WRSTS	= 0x00;
        public byte WREST = 0xB2;
        public byte WRIST = 0xB3;
        //public byte WRPLS	= 0x00;
        //public byte WRSPD	= 0x00;
        //public byte WRSDC	= 0x00;
        public byte WPRMV = 0x80;
        public byte WPRFL = 0x81;
        public byte WPRFH = 0x82;
        public byte WPRUR = 0x83;
        public byte WPRDR = 0x84;
        public byte WPRMG = 0x85;
        public byte WPRDP = 0x86;
        public byte WPRMD = 0x87;
        public byte WPRIP = 0x88;
        public byte WPRUS = 0x89;
        public byte WPRDS = 0x8A;
        public byte WRGPM = 0xBA;
        public byte WRGPD = 0xBB;

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// FTDIオープン処理
        /// FTDI open processing
        /// </summary>
        public int Open()
        {
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            int ret;

            ftStatus = hFTDI.LSI_Open("NPM PCL6115-EV Serial access");

            if (PCL6115EV_on == false)
            {
                if (ftStatus != FTDI.FT_STATUS.FT_OK)
                {
                    ret = -1;
                    PCL6115EV_on = false;
                }
                else
                {
                    ret = 0;
                    PCL6115EV_on = true;
                }
            }
            else
            {
                ret = 0;
            }

            return (ret);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// FTDIクローズ処理
        /// FTDI close processing
        /// </summary>
        public void Close()
        {
            if ((PCL6115EV_on == true) && (PCL6115EV_stop == false))
            {
                hFTDI.LSI_Close();
                PCL6115EV_on = false;
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// Instead of "union"
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        [StructLayout(LayoutKind.Explicit)]
        public struct TBytes
        {
            [FieldOffset(0)]
            public byte Byte0;

            [FieldOffset(1)]
            public byte Byte1;

            [FieldOffset(2)]
            public byte Byte2;

            [FieldOffset(3)]
            public byte Byte3;

            [FieldOffset(0)]
            public uint Word;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// ステータス、サブステータスの読出し
        ///   subS ： 0=ステータス  1=サブステータス
        /// Read Status,Sub Status
        ///   subS ： 0 = Status  1 =Sub Status
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS Read_STATUS(ref uint MainStatus, ref uint SubStatus)
        {
            //---------------------------------------------------------
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;
            uint numBytesRead = 0;

            if ((PCL6115EV_on == true) && (PCL6115EV_stop == false))
            {
                // Main status ------------------------------------------------
                //CS=Low
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x00;
                PileUpBuffer[PileUpindex++] = 0x0B;

                PileUpBuffer[PileUpindex++] = 0x35;//SPI(Out on negative edge, in on negative edge)
                PileUpBuffer[PileUpindex++] = 0x02;//Length L 3byte (X-axsi Only)
                PileUpBuffer[PileUpindex++] = 0x00;//Length H
                PileUpBuffer[PileUpindex++] = 0x11;//command status read
                PileUpBuffer[PileUpindex++] = 0x00;//X-axis dumy
                PileUpBuffer[PileUpindex++] = 0x00;
                //CS=High
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x08;
                PileUpBuffer[PileUpindex++] = 0x0B;

                // Sub status -------------------------------------------------
                //CS=Low
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x00;
                PileUpBuffer[PileUpindex++] = 0x0B;

                PileUpBuffer[PileUpindex++] = 0x35;//SPI(Out on negative edge, in on negative edge)
                PileUpBuffer[PileUpindex++] = 0x02;//Length L 3byte (X-axsi Only)
                PileUpBuffer[PileUpindex++] = 0x00;//Length H
                PileUpBuffer[PileUpindex++] = 0x31;//command sub-status read
                PileUpBuffer[PileUpindex++] = 0x00;//X-axis dumy
                PileUpBuffer[PileUpindex++] = 0x00;
                //CS=High
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x08;
                PileUpBuffer[PileUpindex++] = 0x0B;

                ftStatus = hFTDI.LSI_Write(PileUpBuffer, PileUpindex);

                if (ftStatus == FTDI.FT_STATUS.FT_OK)
                {
                    ftStatus = hFTDI.LSI_Read(ref PileUpBuffer, ref numBytesRead);
                }

                MainStatus = (uint)PileUpBuffer[2] * 256 + (uint)PileUpBuffer[1];
                SubStatus = (uint)PileUpBuffer[5] * 256 + (uint)PileUpBuffer[4];

                PileUpindex = 0;
            }
            else
            {
                ftStatus = FTDI.FT_STATUS.FT_OTHER_ERROR;
            }

            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// レジスタの読出し
        ///   ReadCom ： レジスタリードコマンド
        /// Read Register
        ///   ReadCom ： Register read command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS Read_REG(byte ReadCom, ref uint ReadData)
        {
            //---------------------------------------------------------
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            var Data = new TBytes();
            uint numBytesRead = 0;

            if ((PCL6115EV_on == true) && (PCL6115EV_stop == false))
            {
                //CS=Low
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x00;
                PileUpBuffer[PileUpindex++] = 0x0B;

                PileUpBuffer[PileUpindex++] = 0x35;//SPI(Out on negative edge, in on negative edge)
                PileUpBuffer[PileUpindex++] = 0x05;//Length L 6byte
                PileUpBuffer[PileUpindex++] = 0x00;//Length H
                PileUpBuffer[PileUpindex++] = 0x01;//Write Command
                PileUpBuffer[PileUpindex++] = (byte)(ReadCom);
                PileUpBuffer[PileUpindex++] = 0x00;//X-axis dumy
                PileUpBuffer[PileUpindex++] = 0x00;
                PileUpBuffer[PileUpindex++] = 0x00;
                PileUpBuffer[PileUpindex++] = 0x00;
                //CS=High
                PileUpBuffer[PileUpindex++] = 0x80;
                PileUpBuffer[PileUpindex++] = 0x08;
                PileUpBuffer[PileUpindex++] = 0x0B;

                ftStatus = hFTDI.LSI_Write(PileUpBuffer, PileUpindex);

                if (ftStatus == FTDI.FT_STATUS.FT_OK)
                {
                    ftStatus = hFTDI.LSI_Read(ref PileUpBuffer, ref numBytesRead);
                }

                Data.Byte3 = (byte)PileUpBuffer[5];
                Data.Byte2 = (byte)PileUpBuffer[4];
                Data.Byte1 = (byte)PileUpBuffer[3];
                Data.Byte0 = (byte)PileUpBuffer[2];
                ReadData = Data.Word;

                PileUpindex = 0;
            }
            else
            {
                ftStatus = FTDI.FT_STATUS.FT_OTHER_ERROR;
            }

            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// レジスタの書込みデータとコマンドを積み込む
        ///   WriteData ： 書込みデータ
        ///   WriteCom  ： レジスタライトコマンド
        /// Load register write data and commands
        ///   WriteData : Write data
        ///   WriteCom  : Register write command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Write_REG(byte WriteCom, uint WriteData)
        {
            //---------------------------------------------------------
            var bytes = new TBytes();

            if ((PCL6115EV_on == true) && (PCL6115EV_stop == false))
            {
                //Check Buffer over flow
                if (((PileUpindex + 24) < BUFFER_MAX) && (BuffOver == false))
                {
                    bytes.Word = WriteData; //共用体へ代入  //Assign to union

                    //CS=Low
                    PileUpBuffer[PileUpindex++] = 0x80;
                    PileUpBuffer[PileUpindex++] = 0x00;
                    PileUpBuffer[PileUpindex++] = 0x0B;

                    PileUpBuffer[PileUpindex++] = 0x11;//SPI(Clock Data Bytes Out on -ve Clock Edge MSB First (no Read))
                    PileUpBuffer[PileUpindex++] = 0x05;//Length L 6byte
                    PileUpBuffer[PileUpindex++] = 0x00;//Length H
                    PileUpBuffer[PileUpindex++] = 0x01;//Jsc;
                    PileUpBuffer[PileUpindex++] = WriteCom;
                    PileUpBuffer[PileUpindex++] = bytes.Byte0;//Data 7-0
                    PileUpBuffer[PileUpindex++] = bytes.Byte1;//Data 15-8
                    PileUpBuffer[PileUpindex++] = bytes.Byte2;//Data 23-16
                    PileUpBuffer[PileUpindex++] = bytes.Byte3;//Data 31-24
                    //CS=High
                    PileUpBuffer[PileUpindex++] = 0x80;
                    PileUpBuffer[PileUpindex++] = 0x08;
                    PileUpBuffer[PileUpindex++] = 0x0B;
                }
                else
                {
                    BuffOver = true;
                }
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// コマンドの書込み
        ///   Command ： コマンド
        /// Write command
        ///   Command : command
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void Write_COM(byte Command)
        {
            //---------------------------------------------------------
            if (PCL6115EV_on == true)
            {
                //Check Buffer over flow
                if (((PileUpindex + 8) < BUFFER_MAX) && (BuffOver == false))
                {
                    //CS=Low
                    PileUpBuffer[PileUpindex++] = 0x80;
                    PileUpBuffer[PileUpindex++] = 0x00;
                    PileUpBuffer[PileUpindex++] = 0x0B;

                    PileUpBuffer[PileUpindex++] = 0x11;//SPI(Clock Data Bytes Out on -ve Clock Edge MSB First (no Read))
                    PileUpBuffer[PileUpindex++] = 0x01;//Length L 2byte
                    PileUpBuffer[PileUpindex++] = 0x00;//Length H
                    PileUpBuffer[PileUpindex++] = 0x01;//Jsc;
                    PileUpBuffer[PileUpindex++] = Command;
                    //CS=High
                    PileUpBuffer[PileUpindex++] = 0x80;
                    PileUpBuffer[PileUpindex++] = 0x08;
                    PileUpBuffer[PileUpindex++] = 0x0B;
                }
                else
                {
                    BuffOver = true;
                }
            }
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// 積み込んだデータをUSB送信する
        /// USB loading of loaded data
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public FTDI.FT_STATUS SendUsb()
        {
            //---------------------------------------------------------
            FTDI.FT_STATUS ftStatus = FTDI.FT_STATUS.FT_OK;

            if (PCL6115EV_on == true)
            {
                if (PileUpindex > 0)
                {
                    ftStatus = hFTDI.LSI_Write(PileUpBuffer, PileUpindex);
                }

                PileUpindex = 0;
                BuffOver = false;
            }
            else
            {
                ftStatus = FTDI.FT_STATUS.FT_OTHER_ERROR;
            }
            return (ftStatus);
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// バッファクリア
        /// Buffer clear
        /// </summary>
        /// <param name="description"></param>
        /// <returns></returns>
        public void ClearBuffer()
        {
            //---------------------------------------------------------
            PileUpindex = 0;
            BuffOver = false;
        }

        ////////////////////////////////////////////////////////////////////
        /// <summary>
        /// PCL6115スタータキットの初期設定
        /// Initial setting of PCL6115 starter kit
        /// </summary>
        /// <returns></returns>
        public void InitPCL6115STK()
        {
            if ((PCL6115EV_on == true) && (PCL6115EV_stop == false))
            {
                //---------------------------------------------------------
                //RENV1
                Write_REG(WRENV1, 0x00000002);
                //RENV2
                Write_REG(WRENV2, 0x80001D40);

                SendUsb();
            }
        }
    }
}
